unit Codolex.Database.Query.Interfaces;

interface

uses
  System.SysUtils,
  System.Rtti,
  System.Classes,
  System.Generics.Collections,
  Data.DB,
  Codolex.Collections,
  Codolex.Database.Connection.Interfaces,
  Codolex.Database.Entity.Interfaces,
  Codolex.Database.Runtime.Interfaces;

type
  IDatabaseParam = interface;
  IDatabaseParams = interface;

  IDatabaseQuery = interface
    ['{E987CB00-13BD-480E-BBC4-A9A5F77E1558}']

    function GetSQL: TStrings;
    function GetConnection: IDatabaseConnection;

    function ParamByName(const Name: string): IDatabaseParam;
    function Params: IDatabaseParams;

    function  Open: TDataSet;
    procedure Close;

    procedure Execute;
    function Insert(const PrimaryKeyField: IRuntimeField): TValue;

    property SQL: TStrings read GetSQL;
    property Connection: IDatabaseConnection read GetConnection;
  end;

  IDatabaseParam = interface
    ['{A144B29B-3A96-4EEF-BB68-3006214E334C}']

    function GetAsBoolean: Boolean;
    function GetAsCurrency: Currency;
    function GetAsDateTime: TDateTime;
    function GetAsExtended: Extended;
    function GetAsFloat: Double;
    function GetAsInteger: Integer;
    function GetAsLargeInt: LargeInt;
    function GetAsSmallInt: SmallInt;
    function GetAsStream: TStream;
    function GetAsString: string;
    procedure SetAsBoolean(const Value: Boolean);
    procedure SetAsCurrency(const Value: Currency);
    procedure SetAsDate(const Value: TDateTime);
    procedure SetAsDateTime(const Value: TDateTime);
    procedure SetAsExtended(const Value: Extended);
    procedure SetAsFloat(const Value: Double);
    procedure SetAsInteger(const Value: Integer);
    procedure SetAsLargeInt(const Value: LargeInt);
    procedure SetAsSmallInt(const Value: SmallInt);
    procedure SetAsStream(const Value: TStream);
    procedure SetAsString(const Value: string);
    procedure SetAsTime(const Value: TDateTime);
    procedure SetName(const Value: string);
    function GetName: string;
    procedure SetAsVariant(const Value: Variant);
    function GetAsVariant: Variant;


    property Name: string read GetName write SetName;
    property Value: Variant read GetAsVariant write SetAsVariant;

    property AsFloat: Double read GetAsFloat write SetAsFloat;
    property AsExtended: Extended read GetAsExtended write SetAsExtended;
    property AsCurrency: Currency read GetAsCurrency write SetAsCurrency;

    property AsSmallInt: SmallInt read GetAsSmallInt write SetAsSmallInt;
    property AsInteger: Integer read GetAsInteger write SetAsInteger;
    property AsLargeInt: LargeInt read GetAsLargeInt write SetAsLargeInt;

    property AsBoolean: Boolean read GetAsBoolean write SetAsBoolean;

    property AsString: string read GetAsString write SetAsString;

    property AsDate: TDateTime read GetAsDateTime write SetAsDate;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsTime: TDateTime read GetAsDateTime write SetAsTime;

    property AsStream: TStream read GetAsStream write SetAsStream;
  end;

  IDatabaseParams = interface
    ['{8DCDBC08-3E28-4141-AC62-15CE419E4A65}']

    procedure Add(const Param: IDatabaseParam); overload;
    function Add(const Name: string): IDatabaseParam; overload;

    function ParamByName(const Name: string): IDatabaseParam;
    function HasParam(const Name: string): Boolean;

    function Items: TDictionary<string, IDatabaseParam>;
  end;

  ENoQueryDefined = class(Exception);
  EParamNotFound = class(Exception);

implementation

end.
