unit Codolex.Database.Query.NexusDb;

interface

uses
  Codolex.Database.Query.Interfaces,
  Codolex.Database.Connection.Interfaces,
  Codolex.Database.Runtime.Interfaces,
  System.Classes,
  Data.DB,
  System.Rtti,
  nxdb,
  Codolex.Database.Connection.NexusDb,
  System.SysUtils;

type
  TDatabaseQueryNexusDb = class(TInterfacedObject, IDatabaseQuery)
  private
    FConnection: IDatabaseConnectionNexusDb;
    FQuery: TnxQuery;
    FSQL: TStrings;

    function GetSQL: TStrings;
    function GetConnection: IDatabaseConnection;

    procedure SetQueryText;
  public
    constructor Create(const Connection: IDatabaseConnectionNexusDb);
    procedure BeforeDestruction; override;

    function ParamByName(const Name: string): IDatabaseParam;
    function Params: IDatabaseParams;

    function  Open: TDataSet;
    procedure Close;

    procedure Execute;
    function Insert(const PrimaryKeyField: IRuntimeField): TValue;

    property SQL: TStrings read GetSQL;
    property Connection: IDatabaseConnection read GetConnection;
  end;

implementation

uses
  Codolex.Database.Param.DataDb,
  Codolex.Database.Params,
  nxllTypes;

{ TDatabaseQueryNexusDb }

constructor TDatabaseQueryNexusDb.Create(const Connection: IDatabaseConnectionNexusDb);
begin
  inherited Create;
  FConnection := Connection;

  FQuery := TNxQuery.Create(nil);
  FQuery.Database := Connection.Connection;

  FSQL := TStringList.Create;
end;

procedure TDatabaseQueryNexusDb.BeforeDestruction;
begin
  inherited;

  if FQuery.Active then
    FQuery.Close;

  FreeAndNil(FQuery);
  FreeAndNil(FSQL);
end;

function TDatabaseQueryNexusDb.ParamByName(const Name: string): IDatabaseParam;
begin
  SetQueryText;

  var Param := FQuery.ParamByName(Name);

  Result := TDatabaseParamDataDb.Create(Param);
end;

function TDatabaseQueryNexusDb.Params: IDatabaseParams;
begin
  SetQueryText;

  Result := TDatabaseParams.Create;

  for var i := 0 to FQuery.ParamCount - 1 do
  begin
    var Param := FQuery.Params[i];
    var DatabaseParam := TDatabaseParamDataDb.Create(Param);

    Result.Add(DatabaseParam);
  end;
end;

function TDatabaseQueryNexusDb.Open: TDataSet;
begin
  SetQueryText;

  FQuery.Open;
  Result := FQuery;
end;

procedure TDatabaseQueryNexusDb.Close;
begin
  if FQuery.Active then
    FQuery.Close;
end;

procedure TDatabaseQueryNexusDb.Execute;
begin
  SetQueryText;

  FQuery.ExecSQL;
end;

function TDatabaseQueryNexusDb.Insert(const PrimaryKeyField: IRuntimeField): TValue;
begin
  SetQueryText;

  var IsAutoIncField := Assigned(PrimaryKeyField) and (PrimaryKeyField.FieldType = TFieldType.ftAutoInc);

  if IsAutoIncField then
  begin
    FQuery.ExecSQL;

    var AutoIncValue: TnxWord32;
    FQuery.Database.GetAutoIncValue(PrimaryKeyField.TableName, '', AutoIncValue);

    Result := TValue.From<Cardinal>(AutoIncValue);
  end
  else
  begin
    FQuery.ExecSQL;
    Result := TValue.Empty;
  end;
end;

function TDatabaseQueryNexusDb.GetSQL: TStrings;
begin
  Result := FSQL;
end;

function TDatabaseQueryNexusDb.GetConnection: IDatabaseConnection;
begin
  Result := FConnection;
end;

procedure TDatabaseQueryNexusDb.SetQueryText;
begin
  FQuery.SQL.Text := FSQL.Text;
end;

end.
